<?php

/**
 * @file
 * Settings forms the for Commerce Canada Post module.
 */


/**
 * Form callback: builds the settings form.
 *
 * Used to collect API credentials and enable supported shipping services.
 */
function commerce_canadapost_settings_form($form, &$form_state) {
  $form['sellonline'] = array(
    '#type' => 'fieldset',
    '#title' => t('Canada Post Sell Online Settings'),
    '#collapsible' => TRUE,
  );

  $form['sellonline']['commerce_canadapost_cpcid'] = array(
    '#type' => 'textfield',
    '#title' => t('Retailer ID'),
    '#description' => t('Your Canada Post Sell Online Retailer ID.  See the <a href="@url">Canada Post Sell Online Website</a> for more information.', array(
      '@url' => 'http://sellonline.canadapost.ca/',
    )),
    '#required' => TRUE,
    '#default_value' => variable_get('commerce_canadapost_cpcid', 'CPC_DEMO_XML'),
  );

  $form['sellonline']['commerce_canadapost_origin'] = array(
    '#type' => 'textfield',
    '#title' => t('Origin Postal Code'),
    '#description' => t('The postal code to ship packages from. Overrides the setting in your Sell Online account.'),
    '#default_value' => variable_get('commerce_canadapost_origin', ''),
    '#size' => 7,
    '#maxlength' => 7,
  );

  $date_format = variable_get('commerce_canadapost_delivery_date_format', '');
  if (!in_array($date_format, array_keys(system_get_date_types()))) {
    $date_format = '';
  }
  $form['sellonline']['commerce_canadapost_delivery_date_format'] = array(
    '#type' => 'radios',
    '#title' => t('Delivery Date Estimate'),
    '#description' => t('Select a display format for the estimated delivery date returned by Canada Post. See !url to modify available date formats.', array(
      '!url' => l(t('this page'), 'admin/config/regional/date-time'),
    )),
    '#options' => commerce_canadapost_option_list_date_formats(),
    '#default_value' => $date_format,
  );

  $form['sellonline']['commerce_canadapost_turnaround'] = array(
    '#type' => 'textfield',
    '#states' => array(
      'invisible' => array(
        ':input[name="commerce_canadapost_delivery_date_format"]' => array('value' => FALSE),
      ),
    ),
    '#title' => t('Turnaround Time'),
    '#description' => t('Amount of time to add to the delivery date estimate. Overrides the setting in your Sell Online account.'),
    '#default_value' => variable_get('commerce_canadapost_turnaround', ''),
    '#field_suffix' => t('hours'),
    '#size' => 4,
    '#maxlength' => 4,
  );

  $form['sellonline']['commerce_canadapost_package_count'] = array(
    '#type' => 'select',
    '#title' => t('Package Count'),
    '#description' => t('During checkout, display the number of packages required to ship the order.'),
    '#options' => array(
      'hide'     => t('Do not display'),
      'multiple' => t('Display if more than one'),
      'show'     => t('Always display'),
    ),
    '#default_value' => variable_get('commerce_canadapost_package_count', 'hide'),
  );

  $form['sellonline']['commerce_canadapost_cache_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('Cached rates timeout'),
    '#description' => t('Enter the number of seconds after which cached shipping rates should no longer be considered valid.') . '<br />' . t('Set to 0 during testing to avoid caching failed rate request responses.'),
    '#default_value' => variable_get('commerce_canadapost_cache_timeout', 60),
    '#field_suffix' => t('seconds'),
    '#size' => 4,
    '#maxlength' => 4,
  );

  $form['sellonline']['commerce_canadapost_log'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Log the following messages for debugging'),
    '#options' => array(
      'request' => t('API request messages'),
      'response' => t('API response messages'),
    ),
    '#default_value' => variable_get('commerce_canadapost_log', array()),
  );

  $form['markup'] = array(
    '#type' => 'fieldset',
    '#title' => t('Markup / Handling Fees'),
    '#collapsible' => TRUE,
    '#attached' => array(
      'css' => array(
        drupal_get_path('module', 'commerce_canadapost') . '/theme/commerce_canadapost.admin.css',
      ),
      'js' => array(
        drupal_get_path('module', 'commerce_canadapost') . '/theme/commerce_canadapost.admin.js',
      ),
    ),
  );

  $rate_markup = variable_get('commerce_canadapost_rate_markup', array('type' => 'sellonline', 'amount' => ''));

  // These two fields get combined together with #prefix, #suffix, and css.
  $form['markup']['commerce_canadapost_rate_markup_type'] = array(
    '#type' => 'select',
    '#title' => t('Rate Markup'),
    '#prefix' => '<div class="rate-markup-field">',
    '#options' => array(
      'sellonline'  => t('Use handling fee from Sell Online'),
      'flat'        => t('Flat rate'),
      'package'     => t('Flat rate per package'),
      'percentage'  => t('Percentage of shipping rate'),
    ),
    '#default_value' => $rate_markup['type'],
  );
  $form['markup']['commerce_canadapost_rate_markup'] = array(
    '#type' => 'textfield',
    '#suffix' => '</div>',
    '#description' => t('If this field is left blank, the handling fee in your Sell Online account will be used.'),
    '#field_prefix' => '$',
    '#field_suffix' => '%',
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => $rate_markup['amount'],
  );

  $weight_markup = variable_get('commerce_canadapost_weight_markup', array('weight' => 0, 'unit' => 'g'));

  // These two fields get combined together with #prefix, #suffix, and css.
  $form['markup']['commerce_canadapost_weight_markup'] = array(
    '#type' => 'textfield',
    '#title' => t('Weight Markup'),
    '#prefix' => '<div class="weight-markup-field">',
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => $weight_markup['weight'],
  );
  $form['markup']['commerce_canadapost_weight_markup_unit'] = array(
    '#type' => 'select',
    '#suffix' => '</div>',
    '#description' => t('An amount of weight to add to each package.'),
    // Get an options list for weight units.
    // Reverse the array so that metric comes first (smallest to largest).
    '#options' => array_reverse(physical_weight_unit_options()),
    '#default_value' => $weight_markup['unit'],
  );

  $form['services'] = array(
    '#type' => 'fieldset',
    '#title' => t('Enabled shipping services'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE,
  );

  // Build options array.
  $package_services = commerce_canadapost_service_option_list();

  foreach ($package_services as $optgroup => $optgroup_options) {
    $form['services'][strtr($optgroup, ' ', '-')] = array(
      '#type' => 'checkboxes',
      '#title' => check_plain($optgroup),
      '#options' => $optgroup_options,
      '#default_value' => array_keys(commerce_canadapost_load_services()),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
    '#suffix' => l(t('Cancel'), 'admin/commerce/config/shipping/methods'),
  );

  return $form;
}

/**
 * Validate handler: validates turnaround time and origin postal code.
 */
function commerce_canadapost_settings_form_validate($form, &$form_state) {
  // Check markup = valid percentage or currency amount.
  $rate_markup = $form_state['values']['commerce_canadapost_rate_markup'];
  if ($rate_markup != '') {
    switch ($form_state['values']['commerce_canadapost_rate_markup_type']) {
      case 'sellonline':
        form_set_value($form['markup']['commerce_canadapost_rate_markup'], '', $form_state);
        break;

      case 'percentage':
        // Check for and remove percent sign.
        $rate_markup = str_replace('%', '', $form_state['values']['commerce_canadapost_rate_markup']);
        if (!is_numeric($rate_markup) || ($rate_markup < 0)) {
          form_set_error('commerce_canadapost_rate_markup', t('You must enter a positive numeric value for rate markup / handling percentage.'));
        }
        form_set_value($form['markup']['commerce_canadapost_rate_markup'], $rate_markup, $form_state);
        break;

      default:
        if (!is_numeric($form_state['values']['commerce_canadapost_rate_markup']) || ($rate_markup < 0)) {
          form_set_error('commerce_canadapost_rate_markup', t('You must enter a positive numeric value for rate markup / handling amount.'));
        }
    }
  }

  // Weight markup = positive float value.
  if (!is_numeric($form_state['values']['commerce_canadapost_weight_markup']) || $form_state['values']['commerce_canadapost_weight_markup'] < 0) {
    form_set_error('commerce_canadapost_weight_markup', t('You must enter a positive numeric value for weight markup.'));
  }

  // Turnaround = positive integer.
  if ($form_state['values']['commerce_canadapost_turnaround'] && !ctype_digit($form_state['values']['commerce_canadapost_turnaround'])) {
    form_set_error('commerce_canadapost_turnaround', t('If you enter a value for <em>Turnaround Time</em> it must be a whole number.'));
  }

  // Valid canadian postal code.
  $origin = drupal_strtoupper(trim($form_state['values']['commerce_canadapost_origin']));
  // Canadian postal codes: 6 characters, alternating letter-number,
  // cannot contain DFIOQU, first character cannot be W or Z, space in middle.
  if ($origin && !preg_match('/^[ABCEGHJKLMNPRSTVXY][0-9][ABCEGHJKLMNPRSTVWXYZ][ ]?[0-9][ABCEGHJKLMNPRSTVWXYZ][0-9]$/', $origin)) {
    form_set_error('commerce_canadapost_origin', t('The value you entered for <em>Origin Postal Code</em> is not a valid Canadian postal code.'));
  }
  else {
    // Update value with uppercase version.
    form_set_value($form['sellonline']['commerce_canadapost_origin'], $origin, $form_state);
  }

  // Check if any services marked for deletion have line items depending on
  // them.
  $enabled_services = commerce_canadapost_load_services();
  $all_services = commerce_canadapost_service_option_list();
  foreach ($form_state['values']['services'] as $optgroup => $options) {
    foreach ($options as $key => $value) {
      // Existing service that was unchecked.
      if (isset($enabled_services[$key]) && ($value == '0')) {
        // Check for related line items.
        $line_items = commerce_canadapost_service_line_item_count($key);

        // Line items were found.
        if ($line_items > 0) {
          // Re-tick the box.
          form_set_value($form['services'][$optgroup][$key], $key, $form_state);

          // Display a message why it can't be deleted.
          drupal_set_message(t('The <em>Canada Post @title</em> shipping service has @count shipping line item(s) depending on it and could not be deleted.', array(
            '@title' => check_plain($all_services[strtr($optgroup, '-', ' ')][$key]),
            '@count' => $line_items,
          )), 'error');
        }
      }
    }
  }
}

/**
 * Submit handler: saves settings and enables/deletes selected services.
 */
function commerce_canadapost_settings_form_submit($form, &$form_state) {
  // Save settings.
  variable_set('commerce_canadapost_cpcid', $form_state['values']['commerce_canadapost_cpcid']);
  variable_set('commerce_canadapost_origin', $form_state['values']['commerce_canadapost_origin']);
  variable_set('commerce_canadapost_turnaround', $form_state['values']['commerce_canadapost_turnaround']);
  variable_set('commerce_canadapost_delivery_date_format', $form_state['values']['commerce_canadapost_delivery_date_format']);
  variable_set('commerce_canadapost_package_count', $form_state['values']['commerce_canadapost_package_count']);
  variable_set('commerce_canadapost_cache_timeout', $form_state['values']['commerce_canadapost_cache_timeout']);
  variable_set('commerce_canadapost_log', $form_state['values']['commerce_canadapost_log']);
  variable_set('commerce_canadapost_rate_markup', array(
    'type'  => $form_state['values']['commerce_canadapost_rate_markup_type'],
    'amount' => (float) $form_state['values']['commerce_canadapost_rate_markup'],
  ));
  variable_set('commerce_canadapost_weight_markup', array(
    'weight' => (float) $form_state['values']['commerce_canadapost_weight_markup'],
    'unit'  => $form_state['values']['commerce_canadapost_weight_markup_unit'],
  ));

  // Load the enabled services.
  $enabled_services = commerce_canadapost_load_services();
  $all_services = commerce_canadapost_service_option_list();

  // Loop over the shipping services...
  foreach ($form_state['values']['services'] as $optgroup => $options) {
    foreach ($options as $key => $value) {
      // If a service was marked to be enabled and hasn't been saved yet, save
      // it to the database now.
      if ($key === $value && !isset($enabled_services[$key])) {
        $service = array(
          'name' => $key,
          'title' => $all_services[strtr($optgroup, '-', ' ')][$key],
        );

        commerce_canadapost_service_save($service);
      }
      // Otherwise if a service that is enabled was unchecked, delete it from
      // the database now.
      elseif (isset($enabled_services[$key]) && ($value == '0')) {
        commerce_canadapost_service_delete($key);
      }
    }
  }

  // Clear service cache, rebuild default rules & menu items.
  commerce_canadapost_rebuild_services();

  drupal_set_message(t('Canada Post Shipping configuration saved.'));

  $form_state['redirect'] = 'admin/commerce/config/shipping/methods';
}

/**
 * Builds the form for deleting services.
 */
function commerce_canadapost_service_delete_form($form, &$form_state, $shipping_service) {
  $form_state['shipping_service'] = $shipping_service;

  $form = confirm_form($form,
    t('Are you sure you want to delete the <em>%title</em> service?', array('%title' => $shipping_service['title'])),
    'admin/commerce/config/shipping/services/canadapost',
    '<p>' . t('This action cannot be undone.') . '</p>',
    t('Delete'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit callback for commerce_canadapost_service_delete_form().
 */
function commerce_canadapost_service_delete_form_submit($form, &$form_state) {
  $shipping_service = $form_state['shipping_service'];

  // Check for related line items.
  $line_items = commerce_canadapost_service_line_item_count($shipping_service['name']);

  // Line items were found.
  if ($line_items > 0) {
    // Display a message why it can't be deleted.
    drupal_set_message(t('The <em>Canada Post @title</em> shipping service has @count shipping line item(s) depending on it and could not be deleted.', array(
      '@title' => $shipping_service['title'],
      '@count' => $line_items,
    )), 'error');
  }
  // No line items found, delete service.
  else {
    commerce_canadapost_service_delete($shipping_service['name']);

    drupal_set_message(t('The service <em>%title</em> has been deleted.', array('%title' => $shipping_service['title'])));
    watchdog('commerce_canadapost', 'Deleted service <em>%title</em>.', array('%title' => $shipping_service['title']), WATCHDOG_NOTICE);

    // Clear service cache, rebuild default rules & menu items.
    commerce_canadapost_rebuild_services();
  }

  $form_state['redirect'] = 'admin/commerce/config/shipping/services/canadapost';
}

/**
 * Displays the delete confirmation form for an existing service.
 *
 * @param string $name
 *   The machine-name of the service to delete.
 */
function commerce_canadapost_service_delete_page($name) {
  return drupal_get_form('commerce_canadapost_service_delete_form', commerce_shipping_service_load($name));
}


/**
 * Saves an enabled service.
 *
 * @param array $service
 *   An array containing a 'name', 'title', and 'weight' of the service.
 * @param bool $update
 *   Boolean indicating whether or not the save should be processed as an update
 *   to an existing enabled service.
 *
 * @return int
 *   The return value of the call to drupal_write_record(), FALSE on failure or
 *   SAVED_NEW / SAVED_UPDATED on success.
 */
function commerce_canadapost_service_save($service, $update = FALSE) {
  $op = drupal_write_record('commerce_canadapost_service', $service, $update ? array('name') : array());

  // If insert was successful, notify other modules that a new service was
  // created.
  if ($op !== FALSE) {
    module_invoke_all('commerce_canadapost_service_insert', $service);
  }

  return $op;
}

/**
 * Disables a service by deleting its database row and related rules.
 *
 * @param string $name
 *   The machine name of the package service to delete.
 */
function commerce_canadapost_service_delete($name) {
  $shipping_service = commerce_shipping_service_load($name);

  db_delete('commerce_canadapost_service')
    ->condition('name', $name)
    ->execute();

  rules_config_delete(array('commerce_shipping_service_' . $name));

  // Notify other modules that this service was deleted.
  module_invoke_all('commerce_canadapost_service_delete', $shipping_service);
}

/**
 * Resets shipping service list, rebuilds default rules & menu items.
 */
function commerce_canadapost_rebuild_services() {
  drupal_static_reset('commerce_canadapost_load_services');
  commerce_shipping_services_reset();
  entity_defaults_rebuild();
  rules_clear_cache(TRUE);
  menu_rebuild();
}

/**
 * Counts and returns the number of line items for a specified service.
 */
function commerce_canadapost_service_line_item_count($service) {
  $query = new EntityFieldQuery();

  $query
    ->entityCondition('entity_type', 'commerce_line_item', '=')
    ->fieldCondition('commerce_shipping_service', 'value', $service, '=')
    ->count();

  return $query->execute();
}

/**
 * Returns an options list of Canada Post services.
 *
 * @return array
 *   An associative array keyed by Canada Post product type, which is then
 *   keyed by machine name.
 */
function commerce_canadapost_service_option_list() {
  $services = array(
    t('Domestic') => array(
      'canadapost_dom_regular'          => t('Regular'),
      'canadapost_dom_xpdt'             => t('Expedited'),
      'canadapost_dom_xpresspost'       => t('Xpresspost'),
      'canadapost_dom_pri_courier'      => t('Priority Courier'),
    ),
    t('US') => array(
      'canadapost_us_tracked'           => t('Tracked Packet USA'),
      'canadapost_us_sp_surface'        => t('Small Packets Surface USA'),
      'canadapost_us_sp_air'            => t('Small Packets Air USA'),
      'canadapost_us_xpdt_bus_contract' => t('Expedited US Business Contract'),
      'canadapost_us_xpresspost'        => t('Xpresspost USA'),
      'canadapost_us_pri_world'         => t('Priority Worldwide USA'),
      'canadapost_us_pri_world_pak'     => t('Priority Worldwide PAK USA'),
    ),
    t('International') => array(
      'canadapost_intl_tracked'         => t('Tracked Packet International'),
      'canadapost_intl_sp_surface'      => t('Small Packets Surface International'),
      'canadapost_intl_surface'         => t('Surface International'),
      'canadapost_intl_sp_air'          => t('Small Packets Air International'),
      'canadapost_intl_air'             => t('Air International'),
      'canadapost_intl_xpresspost'      => t('XpressPost International'),
      'canadapost_intl_pri_world'       => t('Priority Worldwide International'),
      'canadapost_intl_pri_world_pak'   => t('Priority Worldwide PAK International'),
    ),
  );

  return $services;
}

/**
 * Returns an option list of date format types.
 *
 * @return array
 *   An associative array keyed by machine name.
 */
function commerce_canadapost_option_list_date_formats() {
  $date_options_list = array('' => t('Do not display'));

  foreach (system_get_date_types() as $type) {
    $date_options_list[$type['type']] = t('<strong>!title:</strong> !format', array(
      '!title' => $type['title'],
      '!format' => format_date(REQUEST_TIME, $type['type']),
    ));
  }

  return $date_options_list;
}
